/* File:      global_gather.c
 * Purpose:   Driver for MPI_Allgather function
 * Input:     None
 * Output:    Random values generated by processes and sum of these
 *            values.
 *
 * Algorithm: 
 *    1. Each process computes a random int x from 0 to MAX_CONTRIB-1
 *    2. Processes call MPI_Allgather() to gather values from all
 *       processes to all other processes
 *    3. All processes print globally gathered values
 *
 * Compile:  mpicc -g -Wall -o global_gather global_gather.c
 * Run:      mpirun -n <number of processes> global_gather
 *
 * Notes:
 *    1. This version has no restriction on the number of processes.
 */
#include <stdio.h>
#include <stdlib.h>
#include "mpi.h"

#define MAX_CONTRIB 20

int Global_gather(int myContrib, int* list, int my_rank, int nproc, 
		  MPI_Comm comm);

int main(void) {
  int nproc, my_rank;
  MPI_Comm comm;
  int i, np;
  int x, *list;

  MPI_Init(NULL, NULL);
  comm = MPI_COMM_WORLD;
  MPI_Comm_size(comm, &nproc);
  MPI_Comm_rank(comm, &my_rank);

  list = (int *) calloc(nproc, sizeof(int));
  srandom(my_rank);
  x = random() % MAX_CONTRIB;
  printf("Proc %d > x = %d\n", my_rank, x);
  fflush(stdout);
  MPI_Barrier(comm);

  MPI_Allgather(&x, 1, MPI_INT, list, 1, MPI_INT, comm);

  for (np = 0; np < nproc; np++){
    if (np == my_rank) {
      printf("rank %3i list: ", np);
      for (i = 0; i < nproc; i++)
	printf("%5i", list[i]);
      printf("\n");
      fflush(stdout);
    }
    MPI_Barrier(comm);
  }

  MPI_Finalize();
  return 0;
}  /* main */
